@@ -111,12 +111,13 @@ class DistUpgradeQuirks(object):
         logging.debug("running Quirks.PreCacheOpen")
 
     # individual quirks handler that run *after* the cache is opened
-    def focalPostInitialUpdate(self):
+    def hirsutePostInitialUpdate(self):
         # PreCacheOpen would be better but controller.abort fails terribly
         """ run after the apt cache is opened the first time """
-        logging.debug("running Quirks.focalPostInitialUpdate")
+        logging.debug("running Quirks.hirsutePostInitialUpdate")
         self._get_from_and_to_version()
         self._test_and_fail_on_i386()
+        self._test_and_fail_on_aufs()
 
         cache = self.controller.cache
         self._test_and_warn_if_ros_installed(cache)
@@ -132,8 +133,8 @@ class DistUpgradeQuirks(object):
         if self._snapstore_reachable:
             self._calculateSnapSizeRequirements()
 
-    def focalPostUpgrade(self):
-        logging.debug("running Quirks.focalPostUpgrade")
+    def hirsutePostUpgrade(self):
+        logging.debug("running Quirks.hirsutePostUpgrade")
         cache = self.controller.cache
         if 'snapd' not in cache:
             logging.debug("package required for Quirk not in cache")
@@ -141,6 +142,9 @@ class DistUpgradeQuirks(object):
         if cache['snapd'].is_installed and \
                 self._snap_list:
             self._replaceDebsAndSnaps()
+        if 'ubuntu-desktop-raspi' in cache:
+            if cache['ubuntu-desktop-raspi'].is_installed:
+                self._replace_fkms_overlay()
 
     # individual quirks handler when the dpkg run is finished ---------
     def PostCleanup(self):
@@ -340,6 +344,28 @@ class DistUpgradeQuirks(object):
                       "with this hardware."))
                 self.controller.abort()
 
+    def _test_and_fail_on_aufs(self):
+        """
+        Test and fail if docker has aufs storage driver available
+        as it is deprecated after 20.04. Even if no containers are
+        currently running aufs, the upgrade could break existing
+        container images.
+        """
+        for aufs_dir in ("/var/snap/docker/common/var-lib-docker/aufs",
+                         "/var/lib/docker/aufs"):
+            if (os.path.exists(aufs_dir)):
+                logging.error("Docker config uses aufs")
+                summary = _("Sorry, this storage driver is not supported "
+                            "in kernels for newer releases")
+                msg = _("There will not be any further Ubuntu releases "
+                        "that provide kernel support for the aufs "
+                        "storage driver.\n\n"
+                        "Please ensure that none of your containers are "
+                        "using the aufs storage driver, remove the directory "
+                        "%s and try again." % aufs_dir)
+                self._view.error(summary, msg)
+                self.controller.abort()
+
     def _test_and_warn_if_vserver(self):
         """
         upstart and vserver environments are not a good match, warn
@@ -812,7 +838,7 @@ class DistUpgradeQuirks(object):
         pattern = re.compile('(.+)-([0-9]+)-(.+)')
         match1 = pattern.match(term1)
         match2 = pattern.match(term2)
-        if match1:
+        if match1 and match2:
             term1 = '%s-%s' % (match1.group(1),
                                match1.group(2))
             term2 = '%s-%s' % (match2.group(1),
@@ -1087,3 +1113,54 @@ class DistUpgradeQuirks(object):
 
                 self._snap_list[snap] = snap_object
         return self._snap_list
+
+    def _replace_fkms_overlay(self, boot_dir='/boot/firmware'):
+        failure_action = (
+            "you may need to replace the vc4-fkms-v3d overlay with "
+            "vc4-kms-v3d in config.txt on your boot partition")
+
+        try:
+            boot_config_filename = os.path.join(boot_dir, 'config.txt')
+            with open(boot_config_filename, 'r', encoding='utf-8') as f:
+                boot_config = f.read()
+        except FileNotFoundError:
+            logging.error("failed to open boot configuration in %s; %s",
+                          boot_config_filename, failure_action)
+            return
+
+        new_config = ''.join(
+            # startswith and replace used to cope with (and preserve) any
+            # trailing d-t parameters, and any use of the -pi4 suffix
+            '# changed by do-release-upgrade (LP: #1923673)\n#' + line +
+            line.replace('dtoverlay=vc4-fkms-v3d', 'dtoverlay=vc4-kms-v3d')
+            if line.startswith('dtoverlay=vc4-fkms-v3d') else
+            # camera firmware disabled due to incompatibility with "full" kms
+            # overlay; without the camera firmware active it's also better to
+            # disable gpu_mem leaving the default (64MB) to allow as much as
+            # possible for the KMS driver
+            '# disabled by do-release-upgrade (LP: #1923673)\n#' + line
+            if line.startswith('gpu_mem=') or line.rstrip() == 'start_x=1' else
+            line
+            for line in boot_config.splitlines(keepends=True)
+        )
+
+        if new_config == boot_config:
+            logging.warning("no fkms overlay or camera firmware line found "
+                            "in %s", boot_config_filename)
+            return
+
+        try:
+            boot_backup_filename = os.path.join(
+                boot_dir, 'config.txt.distUpgrade')
+            with open(boot_backup_filename, 'w', encoding='utf-8') as f:
+                f.write(boot_config)
+        except IOError as exc:
+            logging.error("unable to write boot config backup to %s: %s; %s",
+                          boot_backup_filename, exc, failure_action)
+            return
+        try:
+            with open(boot_config_filename, 'w', encoding='utf-8') as f:
+                f.write(new_config)
+        except IOError as exc:
+            logging.error("unable to write new boot config to %s: %s; %s",
+                          boot_config_filename, exc, failure_action)
