@@ -28,6 +28,9 @@ import mock
 DistUpgradeConfigParser.CONFIG_OVERRIDE_DIR = None
 
 CURDIR = os.path.dirname(os.path.abspath(__file__))
+dpkg = subprocess.Popen(['dpkg', '--print-architecture'],
+                        stdout=subprocess.PIPE)
+ARCH = dpkg.communicate()[0].decode().strip()
 
 
 class TestComponentOrdering(unittest.TestCase):
@@ -120,13 +123,23 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
 
         # now test the result
         #print(open(os.path.join(self.testdir,"sources.list")).read())
-        self._verifySources("""
+        if ARCH in ('amd64', 'i386'):
+            self._verifySources("""
 # main repo
-deb http://archive.ubuntu.com/ubuntu/ gutsy main restricted multiverse universe
-deb http://de.archive.ubuntu.com/ubuntu/ gutsy main restricted multiverse
-deb-src http://archive.ubuntu.com/ubuntu/ gutsy main restricted multiverse
+deb http://archive.ubuntu.com/ubuntu gutsy main restricted multiverse universe
+deb http://archive.ubuntu.com/ubuntu/ gutsy main restricted multiverse
+deb-src http://archive.ubuntu.com/ubuntu gutsy main restricted multiverse
 deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted multiverse
 deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
+""")
+        else:
+            self._verifySources("""
+# main repo
+deb http://ports.ubuntu.com/ubuntu-ports/ gutsy main restricted multiverse universe
+deb http://ports.ubuntu.com/ubuntu-ports/ gutsy main restricted multiverse
+deb-src http://archive.ubuntu.com/ubuntu gutsy main restricted multiverse
+deb http://ports.ubuntu.com/ubuntu-ports/ gutsy-security main restricted multiverse
+deb http://ports.ubuntu.com/ubuntu-ports/ gutsy-security universe
 """)
         # check that the backup file was created correctly
         self.assertEqual(0, subprocess.call(
@@ -159,13 +172,23 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
 
         # now test the result
         #print(open(os.path.join(self.testdir,"sources.list")).read())
-        self._verifySources("""
+        if ARCH in ('amd64', 'i386'):
+            self._verifySources("""
 # main repo
-# deb http://archive.ubuntu.com/ubuntu/ feisty main restricted multiverse universe
-# deb http://de.archive.ubuntu.com/ubuntu/ feisty main restricted multiverse
-# deb-src http://archive.ubuntu.com/ubuntu/ feisty main restricted multiverse
+# deb http://archive.ubuntu.com/ubuntu feisty main restricted multiverse universe
+# deb http://archive.ubuntu.com/ubuntu/ feisty main restricted multiverse
+# deb-src http://archive.ubuntu.com/ubuntu feisty main restricted multiverse
 # deb http://security.ubuntu.com/ubuntu/ feisty-security main restricted
 # deb http://security.ubuntu.com/ubuntu/ feisty-security universe
+""")
+        else:
+            self._verifySources("""
+# main repo
+# deb http://ports.ubuntu.com/ubuntu-ports/ feisty main restricted multiverse universe
+# deb http://ports.ubuntu.com/ubuntu-ports/ feisty main restricted multiverse
+# deb-src http://archive.ubuntu.com/ubuntu feisty main restricted multiverse
+# deb http://ports.ubuntu.com/ubuntu-ports/ feisty-security main restricted
+# deb http://ports.ubuntu.com/ubuntu-ports/ feisty-security universe
 """)
 
     @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController.abort")
@@ -197,6 +220,7 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
         self.assertTrue(mock_get_distro.called)
         self.assertTrue(v.askYesNoQuestion.called)
 
+    @unittest.skipUnless(ARCH in ('amd64', 'i386'), "ports are not mirrored")
     @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
     @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
     def test_double_check_source_distribution_continue(self, mock_get_distro, mock_sourcesListEntryDownloadable):
@@ -204,6 +228,9 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
         test that an upgrade from feisty with a sources.list containing
         hardy asks a question, and if continued, does something.
         """
+        arch = apt_pkg.config.find("APT::Architecture")
+        apt_pkg.config.set("APT::Architecture", "amd64")
+
         shutil.copy(os.path.join(self.testdir, "sources.list.hardy"),
                     os.path.join(self.testdir, "sources.list"))
         apt_pkg.config.set("Dir::Etc::sourcelist", "sources.list")
@@ -237,7 +264,7 @@ deb-src http://uk.archive.ubuntu.com/ubuntu/ hardy main restricted multiverse
 # deb http://security.ubuntu.com/ubuntu/ hardy-security main restricted
 # deb http://security.ubuntu.com/ubuntu/ hardy-security universe
 
-deb http://archive.ubuntu.com/ubuntu/ gutsy main
+deb http://archive.ubuntu.com/ubuntu gutsy main
 
 """)
         # check that the backup file was created correctly
@@ -246,13 +273,17 @@ deb http://archive.ubuntu.com/ubuntu/ gutsy main
              apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".hardy",
              apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".distUpgrade"
              ]))
+        apt_pkg.config.set("APT::Architecture", arch)
 
+    @unittest.skipUnless(ARCH in ('amd64', 'i386'), "ports are not mirrored")
     @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
     @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
     def test_sources_list_inactive_mirror(self, mock_get_distro, mock_sourcesListEntryDownloadable):
         """
         test sources.list rewrite of an obsolete mirror
         """
+        arch = apt_pkg.config.find("APT::Architecture")
+        apt_pkg.config.set("APT::Architecture", "amd64")
         shutil.copy(os.path.join(self.testdir, "sources.list.obsolete_mirror"),
                     os.path.join(self.testdir, "sources.list"))
         apt_pkg.config.set("Dir::Etc::sourcelist", "sources.list")
@@ -282,7 +313,7 @@ deb http://archive.ubuntu.com/ubuntu/ gutsy main
 ##deb-src http://mirror.mcs.anl.gov/ubuntu feisty-proposed main restricted universe multiverse
 # deb-src http://mirror.mcs.anl.gov/ubuntu gutsy-security main restricted universe multiverse # disabled on upgrade to gutsy
 # deb https://example.com/3rd-party/deb/ stable main # disabled on upgrade to gutsy
-deb http://archive.ubuntu.com/ubuntu/ gutsy main
+deb http://archive.ubuntu.com/ubuntu gutsy main
 deb http://archive.ubuntu.com/ubuntu gutsy main restricted universe multiverse # auto generated by ubuntu-release-upgrader
 deb http://archive.ubuntu.com/ubuntu gutsy-updates main restricted universe multiverse # auto generated by ubuntu-release-upgrader
 deb http://archive.ubuntu.com/ubuntu gutsy-security main restricted universe multiverse # auto generated by ubuntu-release-upgrader
@@ -296,11 +327,13 @@ deb-src http://archive.ubuntu.com/ubuntu gutsy-security main restricted universe
              apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".obsolete_mirror",
              apt_pkg.config.find_file("Dir::Etc::sourcelist") + ".distUpgrade"
              ]))
+        apt_pkg.config.set("APT::Architecture", arch)
 
+    @unittest.skipUnless(ARCH in ('amd64', 'i386'), "ports are not mirrored")
     @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
     def test_sources_list_no_template(self, mock_get_distro):
         """
-        test sources.list rewrite when there is no distro template
+        test sources.list rewrite for a mirror when there is no distro template
         """
         shutil.copy(os.path.join(self.testdir, "sources.list.obsolete_mirror"),
                     os.path.join(self.testdir, "sources.list"))
@@ -358,6 +391,7 @@ deb http://security.ubuntu.com/ubuntu gutsy-security main restricted # auto gene
 deb http://archive.canonical.com/ubuntu gutsy partner
 """)
 
+    @unittest.skipUnless(ARCH in ('amd64', 'i386'), "extras was not for ports")
     @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
     @mock.patch("DistUpgrade.DistUpgradeController.get_distro")
     def test_extras_removal(self, mock_get_distro, mock_sourcesListEntryDownloadable):
@@ -409,7 +443,7 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
         # now test the result
         self._verifySources("""
 deb http://ports.ubuntu.com/ubuntu-ports/ gutsy main restricted multiverse universe
-deb-src http://archive.ubuntu.com/ubuntu/ gutsy main restricted multiverse
+deb-src http://archive.ubuntu.com/ubuntu gutsy main restricted multiverse
 
 deb http://ports.ubuntu.com/ubuntu-ports/ gutsy-security main restricted universe multiverse
 """)
@@ -438,7 +472,7 @@ deb http://ports.ubuntu.com/ubuntu-ports/ gutsy-security main restricted univers
         # now test the result
         self._verifySources("""
 deb http://ports.ubuntu.com/ubuntu-ports/ hardy main restricted multiverse universe
-deb-src http://archive.ubuntu.com/ubuntu/ hardy main restricted multiverse
+deb-src http://archive.ubuntu.com/ubuntu hardy main restricted multiverse
 
 deb http://ports.ubuntu.com/ubuntu-ports/ hardy-security main restricted universe multiverse
 """)
@@ -480,6 +514,7 @@ deb-src http://old-releases.ubuntu.com/ubuntu hoary main restricted multiverse
 deb http://old-releases.ubuntu.com/ubuntu hoary-security main restricted universe multiverse
 """)
 
+    @unittest.skipUnless(ARCH in ('amd64', 'i386'), "ports are not mirrored")
     @unittest.skipUnless(url_downloadable(
         "http://us.archive.ubuntu.com/ubuntu", logging.debug),
         "Could not reach mirror")
@@ -508,6 +543,7 @@ deb-src http://us.archive.ubuntu.com/ubuntu precise main restricted multiverse
 deb http://us.archive.ubuntu.com/ubuntu precise-security main restricted universe multiverse
 """)
 
+    @unittest.skipUnless(ARCH in ('amd64', 'i386'), "ports are not mirrored")
     @unittest.skipUnless(url_downloadable(
         "https://mirrors.kernel.org/ubuntu", logging.debug),
         "Could not reach mirror")
@@ -551,12 +587,21 @@ deb https://mirrors.kernel.org/ubuntu bionic-security main restricted universe m
         d.openCache(lock=False)
         res = d.updateSourcesList()
         self.assertTrue(res)
-        self._verifySources("""
+        if ARCH in ('amd64', 'i386'):
+            self._verifySources("""
 # main repo
 deb http://archive.ubuntu.com/ubuntu precise main restricted multiverse universe
 deb-src http://archive.ubuntu.com/ubuntu precise main restricted multiverse
 
 deb http://archive.ubuntu.com/ubuntu precise-security main restricted universe multiverse
+""")
+        else:
+            self._verifySources("""
+# main repo
+deb http://ports.ubuntu.com/ubuntu-ports/ precise main restricted multiverse universe
+deb-src http://archive.ubuntu.com/ubuntu precise main restricted multiverse
+
+deb http://ports.ubuntu.com/ubuntu-ports/ precise-security main restricted universe multiverse
 """)
 
     @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
@@ -577,12 +622,7 @@ deb http://archive.ubuntu.com/ubuntu precise-security main restricted universe m
         self.assertTrue(res)
 
         # now test the result
-        self._verifySources("""
-deb http://archive.ubuntu.com/ubuntu/ gutsy main restricted multiverse universe
-deb-src http://archive.ubuntu.com/ubuntu/ gutsy main restricted multiverse
-
-deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted universe multiverse
-
+        self._verifySourcesLine("""
 deb http://archive.canonical.com/ubuntu gutsy partner
 """)
 
@@ -605,9 +645,10 @@ deb http://archive.canonical.com/ubuntu gutsy partner
         self.assertTrue(res)
 
         # now test the result
-        self._verifySources("""
-deb http://archive.ubuntu.com/ubuntu/ gutsy main restricted multiverse universe
-deb-src http://archive.ubuntu.com/ubuntu/ gutsy main restricted multiverse
+        if ARCH in ('amd64', 'i386'):
+            self._verifySources("""
+deb http://archive.ubuntu.com/ubuntu gutsy main restricted multiverse universe
+deb-src http://archive.ubuntu.com/ubuntu gutsy main restricted multiverse
 
 deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted universe multiverse
 
@@ -617,7 +658,21 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted universe m
 # commercial PPA
 deb https://user:pass@private-ppa.launchpad.net/commercial-ppa-uploaders gutsy main
 """)
+        else:
+            self._verifySources("""
+deb http://ports.ubuntu.com/ubuntu-ports/ gutsy main restricted multiverse universe
+deb-src http://archive.ubuntu.com/ubuntu gutsy main restricted multiverse
 
+deb http://ports.ubuntu.com/ubuntu-ports/ gutsy-security main restricted universe multiverse
+
+# random one
+# deb http://user:pass@private-ppa.launchpad.net/random-ppa gutsy main # disabled on upgrade to gutsy
+
+# commercial PPA
+deb https://user:pass@private-ppa.launchpad.net/commercial-ppa-uploaders gutsy main
+""")
+
+    @unittest.skipUnless(ARCH in ('amd64', 'i386'), "test is not arch specific")
     @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
     def test_apt_cacher_and_apt_bittorent(self, mock_sourcesListEntryDownloadable):
         """
@@ -642,8 +697,8 @@ deb http://localhost:9977/archive.canonical.com/ubuntu gutsy partner
 deb http://localhost:9977/us.archive.ubuntu.com/ubuntu/ gutsy main
 deb http://localhost:9977/archive.ubuntu.com/ubuntu/ gutsy main
 
-deb http://archive.ubuntu.com/ubuntu/ gutsy main restricted multiverse universe
-deb-src http://archive.ubuntu.com/ubuntu/ gutsy main restricted multiverse
+deb http://archive.ubuntu.com/ubuntu gutsy main restricted multiverse universe
+deb-src http://archive.ubuntu.com/ubuntu gutsy main restricted multiverse
 
 deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
 deb http://security.ubuntu.com/ubuntu/ gutsy-security universe
@@ -673,11 +728,7 @@ deb http://ppa.launchpad.net/random-ppa quantal main # ppa of Víctor R. Ruiz (v
         self.assertTrue(res)
 
         # verify it
-        self._verifySources("""
-deb http://archive.ubuntu.com/ubuntu gutsy main restricted
-deb http://archive.ubuntu.com/ubuntu gutsy-updates main restricted
-deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
-# A PPA with a unicode comment
+        self._verifySourcesLine("""
 # deb http://ppa.launchpad.net/random-ppa quantal main # ppa of Víctor R. Ruiz (vrruiz) disabled on upgrade to gutsy
 """)
 
@@ -698,11 +749,19 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
         self.assertTrue(res)
 
         # verify it
-        self._verifySources("""
+        if ARCH in ('amd64', 'i386'):
+            self._verifySources("""
 deb http://192.168.1.1/ubuntu gutsy main restricted
 deb http://192.168.1.1/ubuntu gutsy-updates main restricted
 deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
 deb http://archive.ubuntu.com/ubuntu gutsy-backports main restricted universe multiverse
+""")
+        else:
+            self._verifySources("""
+deb http://192.168.1.1/ubuntu gutsy main restricted
+deb http://192.168.1.1/ubuntu gutsy-updates main restricted
+deb http://ports.ubuntu.com/ubuntu-ports/ gutsy-security main restricted
+deb http://ports.ubuntu.com/ubuntu-ports/ gutsy-backports main restricted universe multiverse
 """)
 
     @mock.patch("DistUpgrade.DistUpgradeController.DistUpgradeController._sourcesListEntryDownloadable")
@@ -725,10 +784,7 @@ deb http://archive.ubuntu.com/ubuntu gutsy-backports main restricted universe mu
         self.assertTrue(mock_sourcesListEntryDownloadable.called)
         self.assertTrue(res)
 
-        self._verifySources("""
-deb http://archive.ubuntu.com/ubuntu gutsy main restricted
-deb http://archive.ubuntu.com/ubuntu gutsy-updates main restricted
-deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
+        self._verifySourcesLine("""
 # deb http://archive.ubuntu.com/ubuntu gutsy-proposed universe main multiverse restricted #Not for humans during development stage of release gutsy
 """)
 
@@ -742,6 +798,15 @@ deb http://security.ubuntu.com/ubuntu/ gutsy-security main restricted
                 "expected entry '%s' in sources.list missing. got:\n'''%s'''" %
                 (l, sources_list))
 
+    def _verifySourcesLine(self, expected):
+        sources_file = apt_pkg.config.find_file("Dir::Etc::sourcelist")
+        with open(sources_file) as f:
+            sources_list = f.read()
+            self.assertTrue(
+                expected in sources_list,
+                "expected entry '%s' in sources.list missing. got:\n'''%s'''" %
+                 (expected.strip("\n"), sources_list))
+
     def _verifySources2Way(self, expected):
         self._verifySources(expected)
         sources_file = apt_pkg.config.find_file("Dir::Etc::sourcelist")
